//---------------------------------------------------------------------------

#ifndef XMLPropsStorageH
#define XMLPropsStorageH
//---------------------------------------------------------------------------

#include <SysUtils.hpp>
#include <Classes.hpp>
#include <ComCtrls.hpp>
#include <Grids.hpp>

#include "DataConverter.h"
#include "XMLStorage.h"
#include "KlenUtils.h"

namespace Klentoolslib
{
enum TPropMode { pmRemove ,  pmRead , pmWrite , pmReadWrite } ;

#define tkUnpublished  -100  

//---------------------------------------------------------------------------
class PACKAGE EXMLPropsStorage : public Exception
{
 public:
  __fastcall EXMLPropsStorage (AnsiString Mess) : Exception(Mess) {} ;
};
//---------------------------------------------------------------------------
class  PACKAGE TXMLPropsStorage : public TComponent
{
public:
  typedef void __fastcall (__closure *TPropPassedEvent )(TObject* Sender , int Prop , AnsiString FullPropName ) ;
  enum TStoreDirect   { sdLifo , sdFifo } ;
  enum TRootType { rtUndefine, rtUnknown, rtForm , rtDataModule } ;
  enum TMethodStoredMode {msmGeneral , msmMenuItem } ;



private:
        Variant            VNull                 ;
        HANDLE             FIOMutex              ;
        TXMLStorage*       FXMLStorage           ;
        TDataConverter*    FDataConvertor        ;
        bool               FDirect               ;
        TStringList*       FStoredProps          ;
        bool               FXMLStorageInitNeeded ;
        AnsiString         FSynchronizeObject    ;
        bool               FActive               ;
        bool               FIsReadNeeded      ;
        bool               FIsWriteNeeded        ;
        TNotifyEvent       FOnBeginRestore       ;
        TNotifyEvent       FOnEndRestore         ;
        TPropPassedEvent   FOnRestorePropPassed  ;
        TNotifyEvent       FOnBeginStore         ;
        TNotifyEvent       FOnEndStore           ;
        TPropPassedEvent   FOnStorePropPassed    ;

        void __fastcall SetActive (bool val)     ;

        AnsiString __fastcall GetFileName() ;
        void __fastcall SetFileName( AnsiString val );

        TXMLDocOptions __fastcall GetOptions () ;
        void __fastcall SetOptions (TXMLDocOptions val) ;

        void __fastcall WriteTTreeNode ( AnsiString StoredNodeTag , TTreeNode* StoredNode ) ;
        void __fastcall ReadTTreeNode (AnsiString RestoredParentNodeTag , TTreeNode* RestoredParentNode , TTreeNodes* TreeNodes) ;

        void __fastcall WriteTMenuItem ( AnsiString ParentTag , TMenuItem* Item , int ItemIndex ) ;
        void __fastcall ReadTMenuItem (  AnsiString ParentTag , TMenuItem* Item , int ItemIndex ) ;

        void __fastcall UnPackPropInfo ( AnsiString DataString , AnsiString& Component , AnsiString& Prop , TPropMode& Mode ) ;
        AnsiString __fastcall PackPropInfo ( AnsiString Component , AnsiString Prop , TPropMode Mode ) ;

protected:
        void __fastcall LoadBoolProperty(TReader *Reader) ;
        void __fastcall StoreBoolProperty(TWriter *Writer);
        void __fastcall LoadStringsProperty(TReader *Reader) ;
        void __fastcall StoreStringsProperty(TWriter *Writer);
        void __fastcall DefineProperties(TFiler *Filer) ;
        bool __fastcall AppWindowHook ( TMessage& Message ) ;
        void virtual __fastcall InitXMLPropsStorage () {} ;

        void __fastcall WriteTMethod ( AnsiString OwnerComponentName , TComponent* Obj , TPropInfo* Info , TMethodStoredMode Mode) ;
        void __fastcall ReadTMethod ( AnsiString OwnerComponentName , TComponent* Obj , TPropInfo* Info , TMethodStoredMode Mode ) ;

        void __fastcall WriteTStringsProp   ( AnsiString OwnerComponentName , TComponent* Obj , TStrings* Prop , TPropInfo* Info ) ;
        void __fastcall ReadTStringsProp   ( AnsiString OwnerComponentName , TComponent* Obj , TStrings* Prop , TPropInfo* Info ) ;

        void __fastcall WriteTComponentProp ( AnsiString OwnerComponentName , TComponent* Obj , TComponent* Prop , TPropInfo* Info ) ;
        void __fastcall ReadTComponentProp ( AnsiString OwnerComponentName , TComponent* Obj , TComponent* Prop , TPropInfo* Info ) ;

        void __fastcall WriteTPictureProp ( AnsiString OwnerComponentName , TComponent* Obj , TPicture* Prop , TPropInfo* Info ) ;
        void __fastcall ReadTPictureProp ( AnsiString OwnerComponentName , TComponent* Obj , TPicture* Prop , TPropInfo* Info ) ;

        void __fastcall WriteTFontProp ( AnsiString OwnerComponentName , TComponent* Obj , TFont* Prop , TPropInfo* Info ) ;
        void __fastcall ReadTFontProp ( AnsiString OwnerComponentName , TComponent* Obj , TFont* Prop , TPropInfo* Info ) ;

        void __fastcall WriteTTreeNodesProp ( AnsiString OwnerComponentName , TComponent* Obj , TTreeNodes* Prop , TPropInfo* Info ) ;
        void __fastcall ReadTTreeNodesProp ( AnsiString OwnerComponentName , TComponent* Obj , TTreeNodes* Prop , TPropInfo* Info ) ;

        void __fastcall WriteTMenuItemProp ( AnsiString OwnerComponentName , TComponent* Obj , TMenuItem* Prop , TPropInfo* Info ) ;
        void __fastcall ReadTMenuItemProp ( AnsiString OwnerComponentName , TComponent* Obj , TMenuItem* Prop , TPropInfo* Info ) ;

        void __fastcall WriteStringGridUnpublishedProp ( AnsiString OwnerComponentName , TStringGrid* Obj , AnsiString PropName ) ;
        void __fastcall ReadStringGridUnpublishedProp ( AnsiString OwnerComponentName , TStringGrid* Obj , AnsiString PropName ) ;


public:
      __property TStringList* StoredProps = { read=FStoredProps , write=FStoredProps};
      __property bool Direct = { read=FDirect , write=FDirect } ;
      __property TXMLStorage* Storage  = { read=FXMLStorage };

      __fastcall TXMLPropsStorage(TComponent* Owner);
      __fastcall ~TXMLPropsStorage() ;

        void __fastcall ReadProps()  ;
        void __fastcall WriteProps() ;

        //     
        void  __fastcall SetPropRecord ( AnsiString Component , AnsiString PropName , TPropMode  Mode , int ListIndex ) ;   // /   
        void  __fastcall GetPropRecord ( AnsiString Component , AnsiString PropName , TPropMode& Mode ) ;               //   
        void  __fastcall GetPropList ( TStringList* Props )  ;                              //      

        __published:
      __property AnsiString FileName  = { read=GetFileName, write=SetFileName };
      __property TXMLDocOptions Options  = { read=GetOptions , write=SetOptions } ;
      __property bool Active = { read=FActive , write=SetActive , default=true} ;


      __property  TNotifyEvent OnBeginRestore  = { read = FOnBeginRestore , write = FOnBeginRestore } ;
      __property  TNotifyEvent OnEndRestore = { read = FOnEndRestore , write = FOnEndRestore } ;
      __property  TPropPassedEvent OnRestorePropPassed = { read = FOnRestorePropPassed , write = FOnRestorePropPassed } ;

      __property  TNotifyEvent OnBeginStore  = { read = FOnBeginStore , write = FOnBeginStore } ;
      __property  TNotifyEvent OnEndStore = { read = FOnEndStore , write = FOnEndStore } ;
      __property  TPropPassedEvent OnStorePropPassed = { read = FOnStorePropPassed , write = FOnStorePropPassed } ;

};


}

using namespace Klentoolslib ;

//---------------------------------------------------------------------------
#endif
